(********************************************************************)
(*																	*)
(*							Pop up objects							*)
(*																	*)
(********************************************************************)

unit GuiMenu;

interface

uses GuiObj,Video,Crt;

type TPopupObject = object
	Height,Width:Integer;
	constructor Init;
	destructor	Done;
    function	Select(x,y:Integer):Integer;
	function	RunPopup(x,y:Integer):Integer;virtual;
end;

type TMenu = object(TPopupObject)
	Items:Integer;
	MenuText:String;
	constructor	Init(Txt:String);
	function	GetItem(n:Integer):String;
	function	RunPopup(x,y:Integer):Integer;virtual;
end;

type PMenu = ^TMenu;

type TComboBox = object(TGuiObject)
	ComboMenu:PMenu;
	Value:Integer;
	constructor	Init(x,y,w,h,IValue:Integer;CMenu:String);
	destructor	Done;
	function	Get:Integer;virtual;
	procedure	Put(var n:Integer);virtual;
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
end;

type TEditBox = object(TPopupObject)
	Text:String;
	constructor Init(Txt:String);
	procedure	Paint(x,y,c:Integer);
	function	RunPopup(x,y:Integer):Integer;virtual;
end;

type PComboBox = ^TComboBox;
	 PEditBox = ^TEditBox;

implementation

uses Graph;

(**************************************************************)
(*	Pop up object. The base class all popups are derived from *)
(**************************************************************)

constructor TPopUpObject.Init;
begin end;

destructor	TPopUpObject.Done;
begin end;

function	TPopUpObject.RunPopup(x,y:Integer):Integer;
begin
	RunPopup := -1;
end;

function 	TMenu.Select(x,y:Integer):Integer;
var m,Size:Integer;
	p:Pointer;
begin
	if x < 0 then x := 0;
	if y < 0 then y := 0;
	if x + Width + 4 >= GetMaxX then x := GetMaxX-4-Width;
	if y + Height + 4 >= GetMaxY then y := GetMaxY-4-Height;

	Size := ImageSize(x-2,y-2,				{ Size of storage for underneath }
					x+Width+2,y+Height+2);
	GetMem(p,Size);						{ Allocate memory for it }
	GetImage(x-2,y-2,x+Width+2,y+Height+2,p^);	{ Save it }
	Box3D(x-2,y-2,x+Width+2,y+Height+1,False);
    m := RunPopup(x,y);
	PutImage(x-2,y-2,p^,NormalPut);		{ Restore the screen }
	FreeMem(p,Size);
	Select := m;
end;

(************************)
(*	Initialise a menu	*)
(************************)

constructor	TMenu.Init(Txt:String);
var s:String;
begin
	MenuText := Txt;					{ Store strings }
	Items := 0;							{ Start at the beginning }
	Width := 0;
	while GetItem(Items+1) <> '' do		{ While values to get.... }
		begin
			Inc(Items);
			s := GetItem(Items);		{ Get the item }
			if TextWidth(s)+16 > Width then Width := TextWidth(s)+16;
		end;
	Height := Items*13;
end;

(****************************)
(*	Extract a menu item		*)
(****************************)

function	TMenu.GetItem(n:Integer):String;
var s:String;
	p:Integer;
begin
	s := MenuText;						{ Copy the string }
	while n > 1 do						{ Rip off n-1 entries }
    	begin
			p := Pos(':',Concat(s,':'));
			Delete(s,1,p);
			Dec(n);
		end;
	p := Pos(':',Concat(s,':'));		{ Remove everything after the colon }
	GetItem := Copy(s,1,p-1);
end;

(************************************)
(*	Pop up and select from a menu	*)
(************************************)

function 	TMenu.RunPopup(x,y:Integer):Integer;
var xc,yc,i:Integer;
	s:String;
	Dummy:Boolean;
begin
	for i := 1 to Items do				{ Draw the menu }
		begin
			yc := (i - 1) * 13 + y;
			Box3D(x,yc,x+Width,yc+12,True);
			s := GetItem(i);SetColor(Black);
			OutTextXY(x+Width div 2-TextWidth(s) div 2,yc+3,s);
		end;

	Dummy := VideoGetClick(xc,yc);		{ Get the input }

	i := -1;
	if  (xc >= x) and (yc >= y) and		{ If in the menu area return item ID }
		(xc < x+Width) and (yc < y+Height) then
									i := (yc - y) div 13 + 1;

    RunPopup := i;
end;

(****************************)
(*	Combo Box GUI Object	*)
(****************************)

constructor	TComboBox.Init(x,y,w,h,IValue:Integer;CMenu:String);
begin
	Value := IValue;
	New(ComboMenu,Init(CMenu));
	TGuiObject.Init(x,y,w,h);
end;

destructor	TComboBox.Done;
begin
	Dispose(ComboMenu,Done);
end;

function	TComboBox.Get:Integer;
begin
	Get := Value;
end;

procedure	TComboBox.Put(var n:Integer);
begin
	Value := n;Paint;
end;

procedure	TComboBox.Paint;
var x,y:Integer;
begin
	Box3D(Left,Top,Right,Bottom,True);
	Box3D(Left+1,Top+1,Left+14,Bottom-1,False);
	SetColor(Black);
	OutTextXY(Left+19,(Top+Bottom-TextHeight(' ')) div 2+1,ComboMenu^.GetItem(Value));
	Rectangle(Left+3,Top+3,Left+11,Bottom-4);
	SetColor(LightGray);
	x := Left+7;y := (Top+Bottom) div 2;
	Line(Left+11,Top+4,Left+11,y);
	SetColor(Black);
	Line(Left+11,y,x,y);Line(x,y,x+2,y+2);Line(x,y,x+2,y-2);
end;

procedure	TComboBox.Click(x,y:Integer);
begin
	x := ComboMenu^.Select(Left+8,(Top+Bottom) div 2);
	if x > 0 then Value := x;
	Paint;
end;

(****************************)
(*		Pop up Edit Box		*)
(****************************)

constructor TEditBox.Init(Txt:String);
begin
	Text := Txt;Height := 84;Width := 360;
end;

procedure	TEditBox.Paint(x,y,c:Integer);
var xc,yc,LineSize:Integer;
	s:String;
begin
	LineSize := (Width - 40) div TextWidth('W');
	s := Concat(Text,'                         ');
	yc := 0;
	Box3D(x+16,y+16,x+Width-16,y+Height-16,True);
	while s <> '' do
		begin
			SetColor(Black);
			OutTextXY(x+20,y+yc*TextHeight('W')* 3 div 2+20,Copy(s,1,LineSize));
			Delete(s,1,LineSize);
			Inc(yc);
		end;
	yc := c div LineSize;
	xc := c - yc * LineSize;
	xc := xc * TextWidth('W') + x + 20 - 1;
	yc := y + yc*TextHeight('W') * 3 div 2 + 20;
	SetColor(Blue);Line(xc,yc-2,xc,yc+TextHeight('W'));
	Line(xc-2,yc-2,xc+2,yc-2);
	Line(xc-2,yc+TextHeight('W'),xc+2,yc+TextHeight('W'));
end;

function	TEditBox.RunPopup(x,y:Integer):Integer;
var Junk,Key:Char;
	Cursor:Integer;
	Complete:Boolean;
begin
	Complete := False;
	Cursor := Length(Text);
	Paint(x,y,Cursor);
	repeat
		Key := ReadKey;
		if Ord(Key) = 0 then Junk := ReadKey;
		Key := UpCase(Key);
		case Ord(Key) of
			8:		if (Text <> '') and (Cursor > 0) then
						begin
							Delete(Text,Cursor,1);
							Dec(Cursor);
							Paint(x,y,Cursor);
						end;

			13,27:	Complete := True;

			else
					if Ord(Key) >= 32 then
						begin
							Insert(Concat(Key,''),Text,Cursor+1);
							Text := Copy(Text,1,255-106);
							Inc(Cursor);
							if Cursor > Length(Text) then
											Cursor := Length(Text);
							Paint(x,y,Cursor);
						end;
			end;
	until Complete;
	RunPopup := 0;
end;

begin
end.
